@@ -826,9 +826,7 @@ class TestTranslation(TestCase):
         Hello
         World</p></xml>'''
         expected = {
-            False: '''<xml><div>TRANSLATED(Hi)</div><p>TRANSLATED(
-
-        Hello
+            False: '''<xml><div>TRANSLATED(Hi)</div><p>\n\n        TRANSLATED(Hello
         World)</p></xml>''',
             True: '''<xml><div>TRANSLATED(Hi)</div><p>TRANSLATED(Hello
         World)</p></xml>'''
@@ -850,6 +848,58 @@ class TestTranslation(TestCase):
             finally:
                 i18n.gettext = default_gettext
 
+    def test_extract_python_inside(self):
+        src = '''<xml><div>${_('hi')}</div><p>
+
+        Hello
+        World</p></xml>'''
+        expected = '''<xml><div>xi</div><p>\n\n        TRANSLATED(Hello
+        World)</p></xml>'''
+
+        # Build translation table
+        messages = {'hi': 'xi'}
+        for _, _, msgid, _ in i18n.extract(BytesIO(src.encode('utf-8')), [], None, {
+            'extract_python': True
+        }):
+            messages[msgid] = 'TRANSLATED(%s)' % msgid
+
+        # Provide a fake translation function
+        default_gettext = i18n.gettext
+        i18n.gettext = lambda s: messages[s]
+        try:
+            perform(src, expected)
+        finally:
+            i18n.gettext = default_gettext
+
+    def test_extract_python_inside_invalid(self):
+        src = '''<xml><div>${_('hi' +}</div></xml>'''
+        try:
+            x = list(i18n.extract(BytesIO(src.encode('utf-8')), [], None, {
+                'extract_python': True
+            }))
+        except KajikiSyntaxError as e:
+            assert "${_('hi' +" in str(e)
+        else:
+            assert False, 'Should have raised'
+
+
+class TestDOMTransformations(TestCase):
+    def test_empty_text_extraction(self):
+        doc = kajiki.xml_template._Parser('<string>', '''<span>  text  </span>''').parse()
+        doc = kajiki.xml_template._DomTransformer(doc, strip_text=False).transform()
+        text_data = [n.data for n in doc.firstChild.childNodes]
+        self.assertEqual(['  ', 'text', '  '], text_data)
+
+    def test_empty_text_extraction_lineno(self):
+        doc = kajiki.xml_template._Parser('<string>', '''<span>
+
+          text
+
+            </span>''').parse()
+        doc = kajiki.xml_template._DomTransformer(doc, strip_text=False).transform()
+        linenos = [n.lineno for n in doc.firstChild.childNodes]
+        self.assertEqual([1, 3, 3], linenos)  # Last node starts on same line as it starts with \n
+
 
 class TestErrorReporting(TestCase):
     def test_syntax_error(self):
